@@ -41,8 +41,10 @@ class DBBase:
         """Returns True if the connection can and should be opened automatically."""
         return not self.is_open
 
-    def log(self, sql: str, params: Optional[Params], duration: float) -> None:
-        """Log out sql and params.
+    def log(
+        self, sql: str, params: Optional[Params], duration: float, error: bool
+    ) -> None:
+        """Hook for logging out sql, params, along with duration and error information.
         Override to do something with this information.
         """
 
@@ -113,14 +115,18 @@ class DBBase:
         """
         await self.auto_open()
         start = time()
+        error = False
 
         try:
             if params:
                 await self.conn.execute(sql, *params)
             else:
                 await self.conn.execute(sql)
+        except:
+            error = True
+            raise
         finally:
-            self.log(sql, params, time() - start)
+            self.log(sql, params, time() - start, error)
 
     async def executemany(self, sql: str, params: Optional[List[Iterable[Any]]] = None):
         """This method passes through to self.conn.execute.
@@ -130,14 +136,18 @@ class DBBase:
         """
         await self.auto_open()
         start = time()
+        error = False
 
         try:
             if params:
                 await self.conn.executemany(sql, params)
             else:
                 await self.conn.executemany(sql)
+        except:
+            error = True
+            raise
         finally:
-            self.log(sql, params, time() - start)
+            self.log(sql, params, time() - start, error)
 
     async def fetch(
         self, sql: Optional[str], params: Optional[Params] = None
@@ -152,14 +162,18 @@ class DBBase:
 
         await self.auto_open()
         start = time()
+        error = False
 
         try:
             if params:
                 return await self.conn.fetch(sql, *params)
 
             return await self.conn.fetch(sql)
+        except:
+            error = True
+            raise
         finally:
-            self.log(sql, params, time() - start)
+            self.log(sql, params, time() - start, error)
 
     async def fetchrow(
         self, sql: Optional[str], params: Optional[Params] = None
@@ -174,14 +188,18 @@ class DBBase:
 
         await self.auto_open()
         start = time()
+        error = False
 
         try:
             if params:
                 return await self.conn.fetchrow(sql, *params)
 
             return await self.conn.fetchrow(sql)
+        except:
+            error = True
+            raise
         finally:
-            self.log(sql, params, time() - start)
+            self.log(sql, params, time() - start, error)
 
     async def select(
         self,
@@ -378,3 +396,8 @@ class DB(DBBase):
 
     async def get_connection_args(self) -> ConnectionArgs:
         return self._conn_args
+
+    def log(
+        self, sql: str, params: Optional[Params], duration: float, error: bool
+    ) -> None:
+        pass
